package lt.ktu.gmj.tasks;

import lt.ktu.gmj.propertySheet.InvalidPropertyException;
import lt.ktu.gmj.propertySheet.SimplePropertyProvider;

class WalrasModelB1Provider extends SimplePropertyProvider 
{
	public WalrasModel _model;
	public WalrasModelB1Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.B1); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B1 = ( (Double) value).doubleValue();
	}
};

class WalrasModelB11Provider extends SimplePropertyProvider 
{
	public WalrasModel _model;
	public WalrasModelB11Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.B11); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B11 = ( (Double) value).doubleValue();
	}
};

class WalrasModelB12Provider extends SimplePropertyProvider
{
	public WalrasModel _model;
	public WalrasModelB12Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.B12); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B12 = ( (Double) value).doubleValue();
	}
};

class WalrasModelB21Provider extends SimplePropertyProvider 
{
	public WalrasModel _model;
	public WalrasModelB21Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.B21); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B21 = ( (Double) value).doubleValue();
	}
};

class WalrasModelB2Provider extends SimplePropertyProvider 
{
	public WalrasModel _model;
	public WalrasModelB2Provider(WalrasModel _m) { _model = _m; }
  
	public Object get() { return new Double(_model.B2); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B2 = ( (Double) value).doubleValue();
	}
};

class WalrasModelB22Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelB22Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.B22); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.B22 = ( (Double) value).doubleValue();
	}
};

class WalrasModelC0Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelC0Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.C0); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.C0 = ( (Double) value).doubleValue();
	}
};

class WalrasModelAProvider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelAProvider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.A); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.A = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ01Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ01Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z01); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z01 = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ11Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ11Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z11); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z11 = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ21Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ21Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z21); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z21 = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ02Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ02Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z02); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z02 = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ12Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ12Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z12); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z12 = ( (Double) value).doubleValue();
	}
};

class WalrasModelZ22Provider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelZ22Provider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Z22); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Z22 = ( (Double) value).doubleValue();
	}
};

class WalrasModelP2StepProvider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelP2StepProvider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Precision); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Precision = ( (Double) value).doubleValue();
	}
};
class WalrasModelP3StepProvider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelP3StepProvider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Precision_Analyzer); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Precision_Analyzer = ( (Double) value).doubleValue();
	}
};

class WalrasModelRegularProvider extends SimplePropertyProvider {
	public WalrasModel _model;
	public WalrasModelRegularProvider(WalrasModel _m) { _model = _m; }

	public Object get() { return new Double(_model.Reg); }

	public void set(Object value) throws InvalidPropertyException 
	{
		_model.Reg = ( (Double) value).doubleValue();
	}
};

class sLayerLogo {
	private static int willDisplay = 1;
	public sLayerLogo() 
	{
		if (willDisplay != 0) 
		{
			System.out.println("Task Walras Model. Created by Povilas Treigys  EISm - 3 VGTU, Vilnius, 2004");
			willDisplay = 0;
		}
	}
};
